/*
 * Copyright (C) 2000-2005 by ETH Zurich
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY ETH ZURICH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ETH ZURICH
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.btnode.ethz.ch
 *
 */

/**
 * \file wsnlab/wsnlab.c
 *
 * \date 16.05.2006
 *
 * \author bo
 *
 * \brief Quick hack library for the WSNLab2006
 *
 */
 
#ifndef _WSNLAB_H_
#define _WSNLAB_H_

// ----------------------------------------------------------------------------
// structs & typedefs
// ----------------------------------------------------------------------------

// forward declaration (does not work)
//typedef struct ccc_packet_st ccc_packet_t;

#include <cc/ccc.h>

typedef struct pkt_element
{
	u_char send_counter;
	u_char recv_counter;
	u_long received_at;
	ccc_packet_t* pkt;
} pkt_element_t;
 
 
// ----------------------------------------------------------------------------
// helper functions
// ----------------------------------------------------------------------------

/**
 * Prints the contents of a packet to the standard output.
 */
void print_packet_short( ccc_packet_t* pkt ); 
 
/**
 * Prints a string representation of the data to stdout (like a hex editor)
 */
void print_data( char* data, u_short length );

/**
 * Computes 2^e
 * 
 * \param e Exponent
 * \return 2^e
 */
u_short pow2(u_short e);

/**
 * Sets the LEDs according to a binary value 0..15
 * 
 * \param i Pattern to set (0..15)
 */
void led_set_binary(u_short i);

/**
 * Sends a packet, using the ccc interface.
 * 
 * \param dest Destination address.
 * \param type Type of the packet.
 * \param data Pointer to the data to send.
 * \param length Length of the data to send.
 * \return ???
 */
int ccc_send_packet( u_short dest, u_char type, u_char* data, u_short length );

/**
 * Beeps for one second, using the BTSense beeper.
 */
void beep(void);

/**
 * Helper function: Initialises the terminal
 */
void term_init(u_long baud, u_char* prompt);

/**
 * Helper function which returns the MAC address stored in the EEPROM (if any)
 */
u_short get_MAC_address_from_EEPROM(void);


/**
 * Helper function which registers the wsnlab commands
 */
void wsnlab_cmds_register_cmds(void);

#endif
