/*
 * Copyright (C) 2000-2005 by ETH Zurich
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY ETH ZURICH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ETH ZURICH
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.btnode.ethz.ch
 *
 */

#include <sys/thread.h>
#include <dev/adc.h>
#include <hardware/btn-hardware.h>
#include <led/btn-led.h> 
#include <teco_ssmall/teco_ssmall.h>
#include <teco_ssmall/micsampler.h>
 
static u_short loudest = 0;

THREAD(MICSAMPLER, arg)
{
    // set a low priority
    NutThreadSetPriority(253);
    
    ADCSetRef(AREF); // AREF = VCC_mV (= 3.3V)
    ADCSetPrescale(ADC_PRESCALE_DIV128); // 62.5kHz sampling @8MHz clock rate
    ADCSetMode(FREE_RUNNING);
    ADCSetChannel(TECO_MICROPHONE_ADC);
    ADCStartConversion();

    for(;;) {
        u_short value;
        // read value
        while(ADCRead(&value))
            NutThreadYield();
        // invert
        value = 1023-value; // 1023 = max value from ADC
        // remember loudest noise
        if(value > loudest)
            loudest = value;
    }
}

u_short mic_read(void){
    u_short value = loudest;
    loudest = 0;
    return value;
}

void mic_set_leds(u_short mic_val){
    if(mic_val <= 1023-200){
        btn_led_clear(0);
        btn_led_clear(1);
        btn_led_clear(2);
        btn_led_clear(3);
    }else if(mic_val <= 1023-50){
        btn_led_set(0);
        btn_led_clear(1);
        btn_led_clear(2);
        btn_led_clear(3);
    }else if(mic_val <= 1023-25){
        btn_led_set(0);
        btn_led_set(1);
        btn_led_clear(2);
        btn_led_clear(3);
    }else if(mic_val <= 1023-10){
        btn_led_set(0);
        btn_led_set(1);
        btn_led_set(2);
        btn_led_clear(3);
    }else if(mic_val <= 1023){
        btn_led_set(0);
        btn_led_set(1);
        btn_led_set(2);
        btn_led_set(3);
    }
}

void mic_init(void){
    NutThreadCreate("micsamp", MICSAMPLER, 0, 512);
}

